
//  Copyright <2022> <Hans de Nivelle>
//
//  This software is released with the 3-clause BSD license, which 
//  is given below: 
// 
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions are met:

//  Redistributions of source code must retain the above copyright notice, 
//  this list of conditions and the following disclaimer:

//  Redistributions in binary form must reproduce the above copyright notice, 
//  this list of conditions and the following disclaimer in the documentation 
//  and/or other materials provided with the distribution.
//  Neither the name of the copyright holder nor the names of its contributors 
//  may be used to endorse or promote products derived from this software 
//  without specific prior written permission.

//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
//  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
//  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
//  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
//  HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
//  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
//  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
//  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
//  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
//  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
//  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.




#ifndef CONFIGURATION_
#define CONFIGURATION_

#include <vector>
#include <iostream>

#include "rhs.h"

#include "turingmachine.h"
#include "reachable.h"

struct configuration
{
   std::vector<char> leftofhead;
   std::vector<char> rightofhead;
   state q;


   // rightofhead is in reversed order because operations at the end of
   // a vector are more efficient.

   // Configuration { a, b, c, d, e, f } with the tape head on d and
   // state q0 will be represented by

   // leftofhead = {a,b,c};
   // rightofhead = {f,e,d};
   // q = q0;

   configuration( ) = delete;
  
   configuration( const state& q, 
                  const std::string& left, const std::string& right );

   configuration( const state& q, const std::string& initial )
      : configuration( q, std::string( ), initial )
   { } 

   configuration& movehead( int dist, char blank );
      // The move always succeeds, because we insert
      // blanks if needed. Negative number means to the left. 
      // Usual notation L,R can be translated as -1, +1.

   bool applicable( const lhs& left, char blank ) const;
      // True if we are in the right state, and have the right
      // next symbol.

   configuration apply( const rhs& right, char blank ) const;
      // We don't check applicability.

   struct cmp
   {
      char blank;
         // cmp must ignore leading and trailing blank symbols, so we 
         // have to know what is blank.
 
      cmp( ) = delete;

      cmp( char blank )
         : blank( blank )
      { }

      int 
      threeway_ignore_leading_blanks( const std::vector< char > & v1,
                                      const std::vector< char > & w2 ) const;
         // 3-way compare, where leading blanks are ignored.

      bool operator( ) ( const configuration& conf1, 
                         const configuration& conf2 ) const;

   };

};

std::ostream& operator << ( std::ostream& out, const configuration& conf );

#endif



