
//  Copyright <2022> <Hans de Nivelle>
//
//  This software is released with the 3-clause BSD license, which 
//  is given below: 
// 
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions are met:

//  Redistributions of source code must retain the above copyright notice, 
//  this list of conditions and the following disclaimer:

//  Redistributions in binary form must reproduce the above copyright notice, 
//  this list of conditions and the following disclaimer in the documentation 
//  and/or other materials provided with the distribution.
//  Neither the name of the copyright holder nor the names of its contributors 
//  may be used to endorse or promote products derived from this software 
//  without specific prior written permission.

//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
//  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
//  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
//  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
//  HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
//  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
//  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
//  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
//  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
//  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
//  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.




// The code below was generated by Maphoon 2022.

// Definition of struct symbol:

#ifndef PARSING_SYMBOL_
#define PARSING_SYMBOL_    1

#include <iostream>
#include <variant>
#include <optional>
#include <stdexcept>

#line 35 "grammar.m"
 #include "../turingmachine.h" 
 #include "location.h" 
 #include <vector> 

namespace parsing { 

   enum symboltype
   {
      sym__recover_, sym_TransitionMap, sym_Transition, sym_LeftRightStat, 
      sym_NewLines, sym_MachineWithInputs, sym_Rhs, sym_STRING, 
      sym_INTEGER, sym_OptInputs, sym_CHAR, sym_COMMA, 
      sym_Lhs, sym_COMMENT, sym_NL, sym_ARROW, 
      sym_BADFILE, sym_State, sym_GARBAGE, sym_WHITESPACE, 
      sym_STARTING, sym_EMPTY, sym_CharacterSet, sym_OptNewLines, 
      sym_TRANSITIONS, sym_DOT, sym_ACCEPTING, sym_BLANK, 
      sym_INPUTS, sym_ALPHABET, sym_Word, sym_EOF, 
      sym_Words, sym_StateSet
   };

   const char* getcstring( symboltype );

   inline std::ostream& operator << ( std::ostream& out, symboltype tp )
      { out << getcstring( tp );  return out; }

   struct symbol
   {
      using attrtype = std::variant < std::monostate, std::set<state>, std::set<char>, 
            state, std::vector<std::string>, lhs, std::string, 
            rhs, char, std::pair<turingmachine, std::vector<std::string> >, int, 
            std::pair<lhs, rhs>, turingmachine::deltatype > ;

      using infotype = location;

      symboltype type;
      std::optional<infotype> info;
      attrtype attr;

      symbol( ) = delete;
      symbol( const symbol& ) = default;
      symbol( symbol&& ) noexcept = default;
      symbol& operator = ( const symbol& ) = default;
      symbol& operator = ( symbol&& ) noexcept = default;
      ~symbol( ) = default;

      symbol( symboltype type,
              const std::optional<infotype> & info )
       : type( type ),
         info( info )
      { }

      symbol( symboltype type,
              const std::optional<infotype> & info,
              attrtype && attr )
       : type( type ),
         info( info ),
         attr( std::move( attr ))
      { }

      symbol( symboltype type,
              const std::optional<infotype> & info,
              const std::set<state> & attr )
       : type( type ),
         info( info ),
         attr( attr )
      { }

      symbol( symboltype type,
              const std::optional<infotype> & info,
              std::set<state> && attr )
       : type( type ),
         info( info ),
         attr( std::move( attr ))
      { }

      symbol( symboltype type,
              const std::optional<infotype> & info,
              const std::set<char> & attr )
       : type( type ),
         info( info ),
         attr( attr )
      { }

      symbol( symboltype type,
              const std::optional<infotype> & info,
              std::set<char> && attr )
       : type( type ),
         info( info ),
         attr( std::move( attr ))
      { }

      symbol( symboltype type,
              const std::optional<infotype> & info,
              const state & attr )
       : type( type ),
         info( info ),
         attr( attr )
      { }

      symbol( symboltype type,
              const std::optional<infotype> & info,
              state && attr )
       : type( type ),
         info( info ),
         attr( std::move( attr ))
      { }

      symbol( symboltype type,
              const std::optional<infotype> & info,
              const std::vector<std::string> & attr )
       : type( type ),
         info( info ),
         attr( attr )
      { }

      symbol( symboltype type,
              const std::optional<infotype> & info,
              std::vector<std::string> && attr )
       : type( type ),
         info( info ),
         attr( std::move( attr ))
      { }

      symbol( symboltype type,
              const std::optional<infotype> & info,
              const lhs & attr )
       : type( type ),
         info( info ),
         attr( attr )
      { }

      symbol( symboltype type,
              const std::optional<infotype> & info,
              lhs && attr )
       : type( type ),
         info( info ),
         attr( std::move( attr ))
      { }

      symbol( symboltype type,
              const std::optional<infotype> & info,
              const std::string & attr )
       : type( type ),
         info( info ),
         attr( attr )
      { }

      symbol( symboltype type,
              const std::optional<infotype> & info,
              std::string && attr )
       : type( type ),
         info( info ),
         attr( std::move( attr ))
      { }

      symbol( symboltype type,
              const std::optional<infotype> & info,
              const rhs & attr )
       : type( type ),
         info( info ),
         attr( attr )
      { }

      symbol( symboltype type,
              const std::optional<infotype> & info,
              rhs && attr )
       : type( type ),
         info( info ),
         attr( std::move( attr ))
      { }

      symbol( symboltype type,
              const std::optional<infotype> & info,
              char attr )
       : type( type ),
         info( info ),
         attr( attr )
      { }

      symbol( symboltype type,
              const std::optional<infotype> & info,
              const std::pair<turingmachine, std::vector<std::string> > & attr )
       : type( type ),
         info( info ),
         attr( attr )
      { }

      symbol( symboltype type,
              const std::optional<infotype> & info,
              std::pair<turingmachine, std::vector<std::string> > && attr )
       : type( type ),
         info( info ),
         attr( std::move( attr ))
      { }

      symbol( symboltype type,
              const std::optional<infotype> & info,
              int attr )
       : type( type ),
         info( info ),
         attr( attr )
      { }

      symbol( symboltype type,
              const std::optional<infotype> & info,
              const std::pair<lhs, rhs> & attr )
       : type( type ),
         info( info ),
         attr( attr )
      { }

      symbol( symboltype type,
              const std::optional<infotype> & info,
              std::pair<lhs, rhs> && attr )
       : type( type ),
         info( info ),
         attr( std::move( attr ))
      { }

      symbol( symboltype type,
              const std::optional<infotype> & info,
              const turingmachine::deltatype & attr )
       : type( type ),
         info( info ),
         attr( attr )
      { }

      symbol( symboltype type,
              const std::optional<infotype> & info,
              turingmachine::deltatype && attr )
       : type( type ),
         info( info ),
         attr( std::move( attr ))
      { }


      void print( std::ostream& out ) const;

      bool has_right_attribute( ) const;

      template< typename T > T& get( )
         { return std::get<T> ( attr ); }
      
      template< typename T > const T& get( ) const &
         { return std::get<T> ( attr ); }
      
      template< typename T > const T& get( ) &&
         { return std::get<T> ( std::move( attr ) ); }
      
   };

   inline
   std::ostream& operator << ( std::ostream& out, const symbol& sym )
      { sym. print( out ); return out; }


}

#endif




