
//  Copyright <2022> <Hans de Nivelle>
//
//  This software is released with the 3-clause BSD license, which 
//  is given below: 
// 
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions are met:

//  Redistributions of source code must retain the above copyright notice, 
//  this list of conditions and the following disclaimer:

//  Redistributions in binary form must reproduce the above copyright notice, 
//  this list of conditions and the following disclaimer in the documentation 
//  and/or other materials provided with the distribution.
//  Neither the name of the copyright holder nor the names of its contributors 
//  may be used to endorse or promote products derived from this software 
//  without specific prior written permission.

//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
//  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
//  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
//  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
//  HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
//  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
//  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
//  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
//  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
//  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
//  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.




#include "tokenizer.h"



void parsing::tokenizer::printstate( std::ostream& out ) const
{
   out << "state of the tokenizer\n";
   out << "   " << inp;
}

#include "DFA.h"

parsing::symbol 
parsing::tokenizer::read( ) 
{
   // static auto cls = buildclassifier( );
   int cls = 71;

restart:
   auto loc = location( inp. line, inp. column );
      // We need the location before reading starts.

   if( !inp. has(1))
      return symbol( sym_EOF, loc );

   if( !inp. good() )
      return symbol( sym_BADFILE, loc );

   std::pair< symboltype, size_t >  p = readandclassify( cls, inp );

   if( p. second == 0 )
   {
      std::string attr; attr. push_back( inp. peek(0) ); 
      inp. commit(1);
      return symbol( sym_GARBAGE, loc, attr ); 
   }

   // Stuff that needs to be ignored:

   if( p. first == sym_WHITESPACE || p. first == sym_COMMENT )
   {
      inp. commit( p. second );
      goto restart;
   }

   // String attribute:

   if( p. first == sym_GARBAGE || p. first == sym_STRING )
   {
      std::string attr = std::string( inp. view( p. second )); 
      inp. commit( p. second );
      return symbol( p. first, loc, std::move( attr )); 
   }

   // Character attribute:

   if( p. first == sym_CHAR )
   { 
      std::string attr = std::string( inp. view( p. second ));
      if( attr. size( ) != 1 )
         throw std::logic_error( "attribute always has length one" );
      inp. commit( p. second );
      return symbol( p. first, loc, attr[0] );
   }

   if( p. first == sym_INTEGER )
   {
      int val = std::stoi( std::string( inp. view( p. second )));
      inp. commit( p. second );
      return symbol( p. first, loc, val );
   }

   // Symbols without attribute:

   inp. commit( p. second );
   return symbol( p. first, loc );
}


void parsing::tokenizer::test( )
{
   auto sym = read( );
   while( sym. type != sym_EOF && sym. type != sym_GARBAGE )
   {
      std::cout << sym << "\n";
      if( !sym. has_right_attribute( ))
         std::cout << "(attribute is wrong)\n";

      sym = read( );
   }
   std::cout << sym << "\n";
}




