
//  Copyright <2022> <Hans de Nivelle>
//
//  This software is released with the 3-clause BSD license, which 
//  is given below: 
// 
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions are met:

//  Redistributions of source code must retain the above copyright notice, 
//  this list of conditions and the following disclaimer:

//  Redistributions in binary form must reproduce the above copyright notice, 
//  this list of conditions and the following disclaimer in the documentation 
//  and/or other materials provided with the distribution.
//  Neither the name of the copyright holder nor the names of its contributors 
//  may be used to endorse or promote products derived from this software 
//  without specific prior written permission.

//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
//  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
//  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
//  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
//  HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
//  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
//  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
//  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
//  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
//  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
//  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.




#include "turingmachine.h"

bool turingmachine::ininputalphabet( const std::string& s ) const 
{
   for( auto ch : s )
   {
      if( !sigma. contains( ch ))
         return false;
   }
   return true;
}

std::set< state > 
turingmachine::states( ) const
{
   std::set< state > res;
   for( const auto& d : delta )
   {
      res. insert( d. first. q );
      res. insert( d. second. q );
   }
   return res;
}

std::set< char > turingmachine::gamma_left( ) const
{
   std::set< char > res;
   for( const auto& d : delta )
      res. insert( d. first. req ); 
   return res; 
}

std::set< char > turingmachine::gamma_right( ) const 
{
   std::set< char > res;
   for( const auto& d : delta )
   {
      for( char c : d. second. repl ) 
         res. insert(c);  
   }
   return res;
}

 
std::ostream& 
operator << ( std::ostream& out, const std::set< char > & set )
{
   out << '{';

   for( auto p = set. begin(); p != set. end(); ++ p )
   {
      if( p != set. begin() )
         out << ", ";
      else
         out << " ";

      out << *p;
   }
   out << " }\n";

   return out; 
}


std::ostream& 
operator << ( std::ostream& out, const std::set< state > & set )
{
   out << '{'; 

   for( auto p = set. begin( ); p != set. end( ); ++ p )
   {
      if( p != set. begin( ))
         out << ", ";
      else
         out << " ";
      out << *p;
   }
   out << " }\n";

   return out;
}


std::ostream& 
operator << ( std::ostream& out, const turingmachine& tm ) 
{
   out << "\n"; 
   out << "Turing Machine:\n";
   out << "   input alphabet =  " << tm. sigma << "\n";
   out << "   q-starting =      " << tm. starting << "\n";
   out << "   q-accepting =     " << tm. accepting << "\n";

   out << "   blank symbol =    " << tm. blank << "\n";
   out << "   delta =\n";
   for( const auto& d : tm. delta )
      out << "      " << d.first << " ---> " << d. second << "\n";

   return out; 
}




